package com.fjhx.mes.service.production.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.utils.Assert;
import com.fjhx.customer.entity.customer.po.Customer;
import com.fjhx.customer.service.customer.CustomerService;
import com.fjhx.item.entity.product.po.ProductClassify;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductClassifyService;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.mes.entity.production.dto.ProduceOrderDetailSelectDto;
import com.fjhx.mes.entity.production.dto.ProductionOrderDetailDto;
import com.fjhx.mes.entity.production.po.ProductionOrder;
import com.fjhx.mes.entity.production.po.ProductionOrderDetail;
import com.fjhx.mes.entity.production.po.ProductionScheduling;
import com.fjhx.mes.entity.production.po.ProductionTaskProgress;
import com.fjhx.mes.entity.production.vo.ProductionOrderDetailVo;
import com.fjhx.mes.entity.production.vo.ProductionSchedulingVo;
import com.fjhx.mes.entity.production.vo.ProductionTaskProgressVo;
import com.fjhx.mes.mapper.production.ProduceOrderDetailMapper;
import com.fjhx.mes.mapper.production.ProductionTaskProgressMapper;
import com.fjhx.mes.service.production.ProduceOrderDetailService;
import com.fjhx.mes.service.production.ProduceOrderService;
import com.fjhx.mes.service.production.ProductionSchedulingService;
import com.fjhx.sale.entity.contract.po.ContractProductBom;
import com.fjhx.sale.entity.contract.vo.ContractProductBomVo;
import com.fjhx.sale.service.contract.ContractProductBomService;
import com.fjhx.sale.service.contract.ContractProductService;
import com.fjhx.tenant.utils.DeptUstil;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 生产订单明细 服务实现类
 * </p>
 *
 * @author
 * @since 2024-01-10
 */
@Service
public class ProduceOrderDetailServiceImpl extends ServiceImpl<ProduceOrderDetailMapper, ProductionOrderDetail> implements ProduceOrderDetailService {

    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private ProductionTaskProgressMapper productionTaskProgressMapper;
    @Autowired
    private ContractProductService contractProductService;
    @Autowired
    private ContractProductBomService contractProductBomService;
    @Autowired
    private ProduceOrderService produceOrderService;
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private ProductionSchedulingService productionSchedulingService;
    @Autowired
    private CustomerService customerService;
    @Autowired
    private ProductClassifyService productClassifyService;

    @Override
    public Page<ProductionOrderDetailVo> getPage(ProduceOrderDetailSelectDto dto) {
        IWrapper<ProductionOrderDetail> wrapper = getPageCommWrapper(dto);

        //在手过滤 未开始 / 生产中的数据
        if (ObjectUtil.isNotEmpty(dto.getIsHand())) {
            wrapper.in("pod.produce_status", 0, 1);
        }

        //是否排程过滤
        if (ObjectUtil.isNotEmpty(dto.getIsScheduling())) {
            wrapper.having("if(count( ps.id )>0,1,0)={0}", dto.getIsScheduling());
        }

        wrapper.groupBy("pod.id");

        Page<ProductionOrderDetailVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        return commPage(page);
    }

    @Override
    public Page<ProductionOrderDetailVo> schedulingTaskPage(ProduceOrderDetailSelectDto dto) {
        IWrapper<ProductionOrderDetail> wrapper = getPageCommWrapper(dto);

//        //开工确认单提交后才能排程
//        wrapper.eq("pod.begin_work_status", 1);

        //任务Id过滤
        wrapper.eq("pod.id", dto.getId());
        //显示当前工序，
        wrapper.eq("ptp.processes_id", dto.getProcessesId());

        //未生产完成
        wrapper.in("pod.produce_status", 0, 1);

        wrapper.groupBy("pod.id");

        //未排程完的任务
//        wrapper.having("IFNULL(SUM( ps.quantity ),0)<pod.quantity");

        Page<ProductionOrderDetailVo> page = this.baseMapper.schedulingTaskPage(dto.getPage(), wrapper);

        List<ProductionOrderDetailVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }

        //获取排程信息
        List<Long> taskIds = records.stream().map(ProductionOrderDetailVo::getId).collect(Collectors.toList());
        List<ProductionSchedulingVo> schedulingLists = productionSchedulingService.getList(IWrapper.getWrapper().in("ps", ProductionScheduling::getTaskId, taskIds)
                .eq("ps", ProductionScheduling::getProcessesId, dto.getProcessesId()));
        Map<Long, List<ProductionSchedulingVo>> schedulingMap = schedulingLists.stream().collect(Collectors.groupingBy(ProductionScheduling::getTaskId));
        for (ProductionOrderDetailVo record : records) {
            //赋值排程信息
            List<ProductionSchedulingVo> schedulingList = schedulingMap.getOrDefault(record.getId(), new ArrayList<>());
            Map<String, ProductionSchedulingVo> collect = schedulingList.stream().collect(Collectors
                    .toMap(item -> DateUtil.format(item.getSchedulingDate(), "yyyy-MM-dd"), Function.identity()));
            record.setSchedulingInfo(collect);
        }

        return commPage(page);
    }

    private IWrapper<ProductionOrderDetail> getPageCommWrapper(ProduceOrderDetailSelectDto dto) {
        IWrapper<ProductionOrderDetail> wrapper = getWrapper();

        //只显示投产的数据
        wrapper.isNotNull("po.produce_time");

        //生产状态
        String produceStatus = dto.getProduceStatus();
        if (ObjectUtil.isNotEmpty(produceStatus)) {
            wrapper.in("pod", ProductionOrderDetail::getProduceStatus, produceStatus.split(","));
        }
        //交期-时间范围过滤
        wrapper.ge("po", ProductionOrder::getDeliveryPeriod, dto.getStaDeliveryPeriod());
        wrapper.le("po", ProductionOrder::getDeliveryPeriod, dto.getEndDeliveryPeriod());
        //下单时间（创建时间）-时间范围过滤
        wrapper.ge("po", ProductionOrder::getCreateTime, dto.getBeginTime());
        wrapper.le("po", ProductionOrder::getCreateTime, dto.getEndTime());
        //关键字
        if (ObjectUtil.isNotEmpty(dto.getKeyword())) {
            //创建人检索
            List<SysUser> list = sysUserService.list(IWrapper.<SysUser>getWrapper().like(SysUser::getNickName, dto.getKeyword()));
            List<Long> uIds = list.stream().map(SysUser::getUserId).collect(Collectors.toList());
            //客户检索
            List<Long> cuIds = customerService.listObject(Customer::getId, q -> q.in(Customer::getName, dto.getKeyword()));

            wrapper.and(q -> q
                    .like("po.code", dto.getKeyword())
                    .or().in("c.create_user", uIds)
                    .or().in("c.buy_corporation_id", cuIds)
                    .or().like("pod.code", dto.getKeyword())
            );
        }

        wrapper.orderByDesc("po", ProductionOrder::getCreateTime);
        wrapper.orderByDesc("pod", ProductionOrderDetail::getId);

        //权限过滤：生产任务
        wrapper.in("po", ProductionOrder::getCompanyId, SecurityUtils.getCompanyIds());
        wrapper.eq("po", ProductionOrder::getCompanyId, dto.getCompanyId());

        //业务公司过滤
        wrapper.eq("c.of_company_id", dto.getContractCompanyId());


        //是否逾期过滤
        if (ObjectUtil.isNotEmpty(dto.getIsOverdue())) {
            wrapper.eq("IF(IFNULL(po.finish_time,NOW())>po.delivery_period,1,0)", dto.getIsOverdue());
        }

        //订单类型检索
        if (ObjectUtil.isNotEmpty(dto.getContractType())) {
            wrapper.eq("c.contract_type", dto.getContractType());
        }

        //欠料状态过滤
        if (ObjectUtil.isNotEmpty(dto.getLackStatus())) {
            wrapper.apply("IFNULL( t2.lackStatus, 0 ) = {0}", dto.getLackStatus());
        }

        //合同id过滤
        wrapper.eq("c.id", dto.getContractId());


        return wrapper;
    }

    private Page<ProductionOrderDetailVo> commPage(Page<ProductionOrderDetailVo> page) {


        List<ProductionOrderDetailVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }

        List<Long> cdIds = records.stream().map(ProductionOrderDetail::getContractDetailId).collect(Collectors.toList());


        //获取产品信息
        List<Long> pIds = records.stream().map(ProductionOrderDetail::getProductId).collect(Collectors.toList());
        List<ProductInfo> productInfoList = productInfoService.list(q -> q.in(ProductInfo::getId, pIds));
        Map<Long, ProductInfo> productMap = productInfoList.stream().collect(Collectors.toMap(ProductInfo::getId, Function.identity()));
        //获取原材料信息
        List<ContractProductBom> rawMaterialBom = contractProductBomService.list(q -> q
                .in(ContractProductBom::getContractProductId, cdIds)
                .eq(ContractProductBom::getType, 1)
        );
        List<ContractProductBomVo> contractProductBomVoList = BeanUtil.copyToList(rawMaterialBom, ContractProductBomVo.class);
        productInfoService.attributeAssign(contractProductBomVoList, ContractProductBomVo::getMaterialId, (item, rawMaterial) -> {
            item.setProductCode(rawMaterial.getCustomCode());
            item.setProductName(rawMaterial.getName());
            item.setProductLength(rawMaterial.getLength());
            item.setProductWidth(rawMaterial.getWidth());
            item.setProductHeight(rawMaterial.getHeight());
            item.setProductColor(rawMaterial.getColor());
        });
//        Map<Long, ContractProductBomVo> rawMaterialMap = contractProductBomVoList.stream().collect(Collectors.toMap(ContractProductBomVo::getContractProductId, Function.identity()));

        //获取生产进度列表
        List<Long> tIds = records.stream().map(ProductionOrderDetail::getId).collect(Collectors.toList());
        List<ProductionTaskProgressVo> productionTaskProgressList = productionTaskProgressMapper.getList(IWrapper.<ProductionTaskProgress>getWrapper()
                .in("ptp.task_id", tIds)
        );
        Map<Long, List<ProductionTaskProgressVo>> productionTaskProgressMap = productionTaskProgressList
                .stream().collect(Collectors.groupingBy(ProductionTaskProgress::getTaskId));

        //对进度列表赋值排程信息
        List<ProductionScheduling> pShList = productionSchedulingService.list(q -> q
                .in(ProductionScheduling::getTaskId, tIds)
                .orderByAsc(ProductionScheduling::getSchedulingDate)
        );
        for (ProductionTaskProgressVo productionTaskProgressVo : productionTaskProgressList) {
            List<ProductionScheduling> schedulingList = pShList.stream()
                    .filter(item ->
                            ObjectUtil.equals(item.getTaskId(), productionTaskProgressVo.getTaskId()) &&
                                    ObjectUtil.equals(item.getProcessesId(), productionTaskProgressVo.getProcessesId())
                    ).collect(Collectors.toList());
            productionTaskProgressVo.setSchedulingList(schedulingList);

            //排程数量求和
            int schedulingCount = schedulingList.stream().map(ProductionScheduling::getQuantity).mapToInt(Integer::intValue).sum();
            productionTaskProgressVo.setSchedulingCount(schedulingCount);
        }


        //获取生产公司信息
        Map<Long, String> companyNameMap = DeptUstil.getDeptNameMap(null);

        //获取产品分类Map
        Map<Long, ProductClassify> productClassifyNameTreeMap = productClassifyService.getNameTreeMap();

        for (ProductionOrderDetailVo record : records) {
            //赋值产品信息
            ProductInfo product = productMap.get(record.getProductId());
            if (ObjectUtil.isNotEmpty(product)) {
                record.setProductCode(product.getCustomCode());
                record.setProductName(product.getName());
                record.setProductUnit(product.getUnit());
                record.setProductSpec(product.getSpec());
                record.setProductLength(product.getLength());
                record.setProductWidth(product.getWidth());
                record.setProductHeight(product.getHeight());
                record.setProductColor(product.getColor());
                record.setProductFrontalTexture(product.getFrontalTexture());
                record.setProductReverseTexture(product.getReverseTexture());
                record.setProductLogoLength(product.getLogoLength());
                record.setProductLogoWidth(product.getLogoWidth());


                //赋值分类
                ProductClassify productClassify = productClassifyNameTreeMap.getOrDefault(product.getProductClassifyId(), new ProductClassify());
                record.setProductClassifyName(productClassify.getName());
                record.setProductClassifyNames(productClassify.getTreeName());
            }

//            //赋值原材料信息
//            ContractProductBomVo rawMaterial = rawMaterialMap.get(record.getContractDetailId());
//            if (ObjectUtil.isNotEmpty(rawMaterial)) {
//                record.setRawMaterialCode(rawMaterial.getProductCode());
//                record.setRawMaterialName(rawMaterial.getProductName());
//            }

            //赋值生产用时
            record.setUsageTime(0L);
            Date produceTime = record.getProduceTime();
            Date finishTime = record.getFinishTime();
            if (ObjectUtil.isNotEmpty(produceTime) && ObjectUtil.isNotEmpty(finishTime)) {
                long usageTime = finishTime.getTime() - produceTime.getTime();
                record.setUsageTime(usageTime / 1000 / 60 / 60);
            }

            //赋值生产进度列表
            record.setProductionTaskProgressList(productionTaskProgressMap.get(record.getId()));

            //赋值生产公司名称
            record.setCompanyName(companyNameMap.get(record.getCompanyId()));
            record.setContractCompanyName(companyNameMap.get(record.getContractCompanyId()));

            //赋值部门名称
            record.setContractDeptName(companyNameMap.get(record.getContractDeptId()));
        }

        //赋值业务员名称
        UserUtil.assignmentNickName(records, ProductionOrderDetailVo::getSaleUserId, ProductionOrderDetailVo::setSaleUserName);

        //赋值客户名称
        customerService.attributeAssign(records, ProductionOrderDetailVo::getCustomerId, (item, customer) -> {
            item.setCustomerName(customer.getName());
        });

        return page;
    }

    @Override
    public ProductionOrderDetailVo detail(ProductionOrderDetailDto dto) {
        ProductionOrderDetailVo detail = BeanUtil.copyProperties(getById(dto.getId()), ProductionOrderDetailVo.class);
        Assert.notEmpty(detail, "查询不到任务明细信息！");

        return detailByIds(Arrays.asList(detail.getId())).get(0);
    }

    @Override
    public List<ProductionOrderDetailVo> detailByIds(List<Long> taskIds) {
        Assert.notEmpty(taskIds, "任务id列表不能为空");

        List<ProductionOrderDetailVo> voList = baseMapper.getPage(getWrapper().in("pod", ProductionOrderDetail::getId, taskIds).groupBy("pod.id"));

        //赋值订单信息
        produceOrderService.attributeAssign(voList, ProductionOrderDetailVo::getProduceOrderId, (detail, order) -> {
            detail.setOrderCode(order.getCode());
            detail.setDeliveryPeriod(order.getDeliveryPeriod());
        });

        //赋值产品信息
        productInfoService.attributeAssign(voList, ProductionOrderDetailVo::getProductId, (detail, product) -> {
            detail.setProductCode(product.getCustomCode());
            detail.setProductName(product.getName());
            detail.setProductUnit(product.getUnit());
            detail.setProductSpec(product.getSpec());
            detail.setProductLength(product.getLength());
            detail.setProductWidth(product.getWidth());
            detail.setProductHeight(product.getHeight());
            detail.setProductColor(product.getColor());
        });
        //赋值产品备注
        contractProductService.attributeAssign(voList, ProductionOrderDetailVo::getContractDetailId, (detail, product) -> {
            detail.setProductRemark(product.getRemark());
        });

        //获取bom信息
        List<Long> cpIds = voList.stream().map(ProductionOrderDetailVo::getContractDetailId).collect(Collectors.toList());
        List<ContractProductBom> cpBomList = contractProductBomService.list(q -> q.in(ContractProductBom::getContractProductId, cpIds));
        List<ContractProductBomVo> cpBomVoList = BeanUtil.copyToList(cpBomList, ContractProductBomVo.class);
        productInfoService.attributeAssign(cpBomVoList, ContractProductBomVo::getMaterialId, (item, product1) -> {
            item.setProductCode(product1.getCustomCode());
            item.setProductName(product1.getName());

            item.setProductLength(product1.getLength());
            item.setProductWidth(product1.getWidth());
            item.setProductHeight(product1.getHeight());
            item.setProductColor(product1.getColor());
        });
        Map<Long, List<ContractProductBomVo>> bomMap = cpBomVoList.stream().collect(Collectors.groupingBy(ContractProductBom::getContractProductId));
        //获取原材料
        Map<Long, List<ContractProductBomVo>> rawBomMap = cpBomVoList.stream()
                .filter(item -> Objects.equals(item.getType(), 1))
                .collect(Collectors.groupingBy(ContractProductBom::getContractProductId));
        //其他BOM
        Map<Long, List<ContractProductBomVo>> otherBomMap = cpBomVoList.stream()
                .filter(item -> !Objects.equals(item.getType(), 1))
                .collect(Collectors.groupingBy(ContractProductBom::getContractProductId));

        //获取生产进度列表
        List<ProductionTaskProgressVo> productionTaskProgressList = productionTaskProgressMapper.getList(IWrapper.<ProductionTaskProgress>getWrapper()
                .in("ptp.task_id", taskIds)
        );
        Map<Long, List<ProductionTaskProgressVo>> ptpMap = productionTaskProgressList.stream().collect(Collectors.groupingBy(ProductionTaskProgress::getTaskId));

        for (ProductionOrderDetailVo detail : voList) {
            //赋值原材料信息
            List<ContractProductBomVo> rawMaterialBomList = rawBomMap.get(detail.getContractDetailId());
            if (ObjectUtil.isNotEmpty(rawMaterialBomList)) {

                String productCode = rawMaterialBomList.stream().map(item -> item.getProductCode()).distinct().collect(Collectors.joining("，"));
                String productName = rawMaterialBomList.stream().map(item -> item.getProductName()).distinct().collect(Collectors.joining("，"));
                String productColor = rawMaterialBomList.stream().map(item -> item.getProductColor()).distinct().collect(Collectors.joining("，"));

                detail.setRawMaterialCode(productCode);
                detail.setRawMaterialName(productName);
                detail.setProductColor(productColor);

//                detail.setRawMaterialLength(rawMaterialBomList.getProductLength());
//                detail.setRawMaterialWidth(rawMaterialBomList.getProductWidth());
//                detail.setRawMaterialHeight(rawMaterialBomList.getProductHeight());
            }
            //赋值BOM信息
            detail.setContractProductBomList(otherBomMap.get(detail.getContractDetailId()));

            //赋值BOM信息
            detail.setContractProductAllBomList(bomMap.get(detail.getContractDetailId()));

            //赋值生产进度列表
            detail.setProductionTaskProgressList(ptpMap.get(detail.getId()));
        }

        //赋值业务员名称
        UserUtil.assignmentNickName(voList, ProductionOrderDetailVo::getSaleUserId, ProductionOrderDetailVo::setSaleUserName);

        //赋值客户名称
        customerService.attributeAssign(voList, ProductionOrderDetailVo::getCustomerId, (item, customer) -> {
            item.setCustomerName(customer.getName());
        });

        return voList;
    }

    @Override
    public void editRemark(ProductionOrderDetailDto dto) {
        Assert.notEmpty(dto.getId(), "生产任务id不能为空！");

        this.update(q -> q
                .eq(ProductionOrderDetail::getId, dto.getId())
                .set(ProductionOrderDetail::getRemark, dto.getRemark())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void editOtherProdRemark(ProductionOrderDetailDto dto) {
        Assert.notEmpty(dto.getId(), "生产任务id不能为空！");

        this.update(q -> q
                .eq(ProductionOrderDetail::getId, dto.getId())
                .set(ProductionOrderDetail::getOtherProdRemark, dto.getOtherProdRemark())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public JSONObject taskCount(ProduceOrderDetailSelectDto dto) {
        dto.setIsHand(null);

        //完成任务查询
        ProduceOrderDetailSelectDto completedCountDto = BeanUtil.copyProperties(dto, ProduceOrderDetailSelectDto.class);
        completedCountDto.setProduceStatus("2,5,10");
        //在手任务查询
        ProduceOrderDetailSelectDto onHandCountDto = BeanUtil.copyProperties(dto, ProduceOrderDetailSelectDto.class);
        onHandCountDto.setIsHand(1);

        //单个工序查询
        String dayDate = DateUtil.format(dto.getSchedulingDate(), "yyyy-MM-dd");
        IWrapper<ProductionOrderDetail> pageCommWrapper = getPageCommWrapper(dto);
        List<ProductionSchedulingVo> productionSchedulingList = baseMapper.getProcessesScheduling(dayDate, pageCommWrapper);

        JSONObject json = new JSONObject();
        json.put("onHandCount", getPage(onHandCountDto).getTotal());//在手任务(没排程的任务数)
        json.put("completedCount", getPage(completedCountDto).getTotal());
        json.put("allCount", getPage(dto).getTotal());//总任务数
        json.put("balanceCount", json.get("allCount"));
        for (ProductionScheduling productionScheduling : productionSchedulingList) {
            json.put(productionScheduling.getProcessesId().toString(), productionScheduling.getQuantity());
        }
        return json;
    }

    /**
     * 报工锁定或解锁
     */
    @Override
    public void reportLockOrUnlock(ProduceOrderDetailSelectDto dto) {
        this.update(q -> q
                .eq(ProductionOrderDetail::getId, dto.getId())
                .setSql("report_lock = if(report_lock=1,0,1)")
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void reportLockOrUnlockAll(ProductionOrderDetailDto dto) {
        this.update(q -> q
                .eq(ProductionOrderDetail::getCompanyId, SecurityUtils.getCompanyId())
                .in(ProductionOrderDetail::getProduceStatus, 0, 1, 2)
                .set(ProductionOrderDetail::getReportLock, dto.getReportLock())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @DSTransactional
    @Override
    public void editBeginWork(ProductionOrderDetailDto dto) {
        this.updateById(dto);
    }

}
