package com.fjhx.oa.service.user.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.utils.Assert;
import com.fjhx.file.entity.FileInfoVo;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.oa.entity.user.dto.UserSalaryManageDto;
import com.fjhx.oa.entity.user.dto.UserSalaryManageSelectDto;
import com.fjhx.oa.entity.user.po.UserSalary;
import com.fjhx.oa.entity.user.po.UserSalaryDetail;
import com.fjhx.oa.entity.user.po.UserSalaryManage;
import com.fjhx.oa.entity.user.vo.UserSalaryDetailVo;
import com.fjhx.oa.entity.user.vo.UserSalaryManageVo;
import com.fjhx.oa.entity.user.vo.UserSalaryVo;
import com.fjhx.oa.mapper.user.UserSalaryManageMapper;
import com.fjhx.oa.service.user.UserSalaryDetailService;
import com.fjhx.oa.service.user.UserSalaryManageService;
import com.fjhx.oa.service.user.UserSalaryService;
import com.fjhx.tenant.utils.DeptUstil;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 员工薪资管理 服务实现类
 * </p>
 *
 * @author _
 * @since 2024-06-11
 */
@Service
public class UserSalaryManageServiceImpl extends ServiceImpl<UserSalaryManageMapper, UserSalaryManage> implements UserSalaryManageService {

    @Resource
    private UserSalaryDetailService userSalaryDetailService;
    @Resource
    private UserSalaryService userSalaryService;

    @Override
    public Page<UserSalaryManageVo> getPage(UserSalaryManageSelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        Page<UserSalaryManageVo> page = baseMapper.getPage(dto.getPage(), wrapper);
        List<UserSalaryManageVo> records = page.getRecords();
        setInfo(records);
        return page;
    }

    @Override
    public UserSalaryManageVo detail(Long userId) {
        Assert.notEmpty(userId, "用户Id不能为空！");
        UserSalaryManageVo detail = baseMapper.getDetail(userId);
        if (ObjectUtil.isEmpty(detail)) {
            throw new ServiceException("查询不到用户薪资信息！");
        }
        setInfo(Collections.singletonList(detail));
        return detail;
    }

    @DSTransactional
    @Override
    public synchronized void edit(UserSalaryManageDto dto) {
        Assert.notEmpty(dto.getUserId(), "用户Id不能为空！");
        Integer dataVer = 0;
        //先处理历史版本数据
        List<UserSalaryDetail> oldList = userSalaryDetailService.list(q -> q
                .eq(UserSalaryDetail::getUserId, dto.getUserId())
                .eq(UserSalaryDetail::getIsCurrentVer, 1)
        );
        if (ObjectUtil.isNotEmpty(oldList)) {
            for (UserSalaryDetail userSalaryDetail : oldList) {
                dataVer = userSalaryDetail.getDataVer();
                userSalaryDetail.setIsCurrentVer(0);
            }
            userSalaryDetailService.updateBatchById(oldList);
        }
        //保存历史版本
        UserSalary oldUserSalary = userSalaryService.getOne(q -> q.eq(UserSalary::getUserId, dto.getUserId()).eq(UserSalary::getIsCurrentVer, 1));
        if (ObjectUtil.isNotEmpty(oldUserSalary)) {
            UserSalary userSalary = BeanUtil.copyProperties(this.getById(dto.getUserId()), UserSalary.class);
            userSalary.setId(oldUserSalary.getId());
            userSalary.setIsCurrentVer(0);
            userSalaryService.updateById(userSalary);
        }

        //再处理新数据
        dataVer++;//版本号+1
        this.updateById(dto);
        ObsFileUtil.editFile(dto.getFileList(), dto.getUserId(), 10);

        UserSalary newUserSalary = BeanUtil.copyProperties(dto, UserSalary.class);
        newUserSalary.setDataVer(dataVer);
        newUserSalary.setIsCurrentVer(1);
        userSalaryService.save(newUserSalary);
        ObsFileUtil.copyFileAndSave(dto.getFileList(), newUserSalary.getId(), 10);

        List<UserSalaryDetail> userSalaryDetailList = dto.getUserSalaryDetailList();
        for (UserSalaryDetail userSalaryDetail : userSalaryDetailList) {
            userSalaryDetail.setId(null);
            userSalaryDetail.setUserSalaryId(newUserSalary.getId());
            userSalaryDetail.setUserId(dto.getUserId());
            userSalaryDetail.setDataVer(dataVer);
            userSalaryDetail.setIsCurrentVer(1);
        }
        userSalaryDetailService.saveBatch(userSalaryDetailList);
    }

    @Override
    public List<UserSalaryVo> getHistoryVerList(Long userId) {
        Assert.notEmpty(userId, "用户id不能为空！");
        List<UserSalaryVo> list = userSalaryService.getList(q -> q
                .eq(UserSalary::getUserId, userId)
                .eq(UserSalary::getIsCurrentVer, 0)
                .orderByDesc(UserSalary::getDataVer)
        );
        if (ObjectUtil.isEmpty(list)) {
            return list;
        }

        //赋值明细
        List<Long> ids = list.stream().map(UserSalary::getId).collect(Collectors.toList());
        List<UserSalaryDetailVo> userSalaryDetailVos = userSalaryDetailService.getList(IWrapper.getWrapper()
                .in(UserSalaryDetail::getUserSalaryId, ids)
        );
        Map<Long, List<UserSalaryDetailVo>> collect = userSalaryDetailVos.stream().collect(Collectors.groupingBy(UserSalaryDetail::getUserSalaryId));
        for (UserSalaryVo userSalary : list) {
            userSalary.setUserSalaryDetailList(collect.getOrDefault(userSalary.getId(), new ArrayList<>()));
        }

        UserUtil.assignmentNickName(list, UserSalary::getUserId, UserSalaryVo::setUserName);

        return list;
    }

    private void setInfo(List<UserSalaryManageVo> list) {
        if (ObjectUtil.isEmpty(list)) {
            return;
        }
        List<Long> userIds = list.stream().map(UserSalaryManage::getUserId).collect(Collectors.toList());
        Map<Long, List<UserSalaryDetail>> usdMap = userSalaryDetailService.mapKGroup(UserSalaryDetail::getUserId, q -> q
                .in(UserSalaryDetail::getUserId, userIds)
                .eq(UserSalaryDetail::getIsCurrentVer, 1)
        );

        Map<Long, List<FileInfoVo>> fileMap = ObsFileUtil.getFileMap(userIds, 10);
        Map<Long, String> deptNameMap = DeptUstil.getDeptNameMap();
        for (UserSalaryManageVo userSalaryManageVo : list) {
            userSalaryManageVo.setUserSalaryDetailList(usdMap.getOrDefault(userSalaryManageVo.getUserId(), new ArrayList<>()));
            userSalaryManageVo.setFileList(fileMap.getOrDefault(userSalaryManageVo.getUserId(), new ArrayList<>()));
            userSalaryManageVo.setCompanyName(deptNameMap.get(userSalaryManageVo.getCompanyId()));
            userSalaryManageVo.setDeptName(deptNameMap.get(userSalaryManageVo.getDeptId()));
        }

        //赋值部门名称树
        DeptUstil.assignmentDeptNames(list, UserSalaryManageVo::getDeptId, (item, nameList) -> {
            item.setDeptNames(nameList.stream().collect(Collectors.joining("=>")));
        });

    }

}
